/*
 * Copyright (c) 2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include "evilwm.h"

#define OPAQUE  0xffffffff

void
settrans(Client *c, double t)
{
	Atom wm_opacity = XInternAtom(dpy, "_NET_WM_WINDOW_OPACITY", False);
	Atom actual;
	unsigned long n, left;
	unsigned long opacity, current_opacity;
	unsigned char *data;
	int format;

	XGetWindowProperty(dpy, c->parent, wm_opacity,
	    0L, 1L, False, XA_CARDINAL, &actual, &format, &n, &left,
	    &data);
	if (data != None)
		current_opacity = *(unsigned long *)data & 0xffffffffUL;
	else
		current_opacity = OPAQUE;

	opacity = (unsigned int)(t * OPAQUE) & 0xffffffffUL;;
	/* Avoid un-needed property change */
	if (opacity == current_opacity)
		return;

	/* un-hide completely translucent window */
	if (current_opacity == 0)
		unhide(c, False);

	if (opacity == OPAQUE)
		/* don't handle fully opaque windows as transparent */
		XDeleteProperty(dpy, c->parent, wm_opacity);
	else
		/* Regular case */
		XChangeProperty(dpy, c->parent, wm_opacity,
		    XA_CARDINAL, 32, PropModeReplace, 
		    (unsigned char *)&opacity, 1L);
	/* hide fully transparent windows */
	if (opacity == 0)
		hide(c);
}

void
fade(Client *c, int flag)
{
	int i;
	double d;
	
	for (i = 0; i < 25; i++) {
		d = (double)i/24.0;
		if (flag) 
			d = 1.0 - d;
		settrans(c, d);
		usleep(20000);
	}
	if (!flag) {
		XMapRaised(dpy, c->parent);
		client_update_current(current, c);
	}

}

void
fade2(Client *c1, Client *c2)
{
	int i;
	double d;
	for (i = 0; i < 25; i++) {
		d = (double)i/24.0;
		settrans(c1, d);
		settrans(c2, 1.0 - d);
		usleep(20000);
	}
	XMapRaised(dpy, c1->parent);
	client_update_current(current, c1);
}
